@@ -28,6 +28,7 @@
 #include <gtk/gtktable.h>
 #include <libgnomedb/libgnomedb.h>
 #include "dsn-properties-dialog.h"
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 enum {
 	REVERT_BUTTON,
@@ -38,12 +39,12 @@ enum {
 static GdaClient *db_client = NULL;
 
 static void
-save_dsn (GnomeDbDsnConfig *props)
+save_dsn (GnomeDbDsnEditor *props)
 {
 	GdaDataSourceInfo *dsn_info;
 
-	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG (props));
-	dsn_info = gnome_db_dsn_config_get_info (GNOME_DB_DSN_CONFIG (props));
+	g_return_if_fail (GNOME_DB_IS_DSN_EDITOR (props));
+	dsn_info = gnome_db_dsn_editor_get_dsn (GNOME_DB_DSN_EDITOR (props));
 	if (dsn_info) {
 		gda_config_save_data_source_info (dsn_info);
 		gda_data_source_info_free (dsn_info);
@@ -51,12 +52,12 @@ save_dsn (GnomeDbDsnConfig *props)
 }
 
 static void
-dsn_changed_cb (GnomeDbDsnConfig *config, gpointer user_data)
+dsn_changed_cb (GnomeDbDsnEditor *config, gpointer user_data)
 {
 	gboolean changed;
 	GdaDataSourceInfo *dsn_info , *old_dsn_info;
 
-	dsn_info = gnome_db_dsn_config_get_info (config);
+	dsn_info = gnome_db_dsn_editor_get_dsn (config);
 	old_dsn_info = g_object_get_data (G_OBJECT (user_data), "old_dsn_info");
 
 	changed = !gda_data_source_info_equal (dsn_info, old_dsn_info);
@@ -70,23 +71,23 @@ dsn_changed_cb (GnomeDbDsnConfig *config, gpointer user_data)
 GdaConnection *
 do_connect_dialog_for_dsn (const gchar *dsn)
 {
-	GtkWidget *dialog, *login;
+	GtkWidget *dialog;
 	gchar *title;
 	GdaConnection *cnc = NULL;
 
 	title = g_strdup_printf (_("Login for %s"), dsn);
 	dialog = gnome_db_login_dialog_new (title);
-	login = gnome_db_login_dialog_get_login_widget (GNOME_DB_LOGIN_DIALOG (dialog));
-	gnome_db_login_set_dsn (GNOME_DB_LOGIN (login), dsn);
+	GnomeDbLogin* login = gnome_db_login_dialog_get_login_widget (GNOME_DB_LOGIN_DIALOG (dialog));
+	gnome_db_login_set_dsn (login, dsn);
 
 	if (gnome_db_login_dialog_run (GNOME_DB_LOGIN_DIALOG (dialog))) {
 		if (!db_client) 
 			db_client = gda_client_new ();
 
 		cnc = gda_client_open_connection (db_client,
-						  gnome_db_login_get_dsn (GNOME_DB_LOGIN (login)),
-						  gnome_db_login_get_username (GNOME_DB_LOGIN (login)),
-						  gnome_db_login_get_password (GNOME_DB_LOGIN (login)),
+						  gnome_db_login_get_dsn (login),
+						  gnome_db_login_get_username (login),
+						  gnome_db_login_get_password (login),
 						  0, NULL);
 	}
 
@@ -120,7 +121,9 @@ dsn_properties_dialog (GtkWindow *parent, const gchar *dsn)
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), REVERT_BUTTON, FALSE);
 	g_object_set_data (G_OBJECT (dialog), "old_dsn_info", old_dsn_info);
 
-	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	str = gnome_db_gbr_get_icon_path ("gnome-db.png");
+	icon = gdk_pixbuf_new_from_file (str, NULL);
+	g_free (str);
 	if (icon) {
 		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
 		g_object_unref (icon);
@@ -152,8 +155,8 @@ dsn_properties_dialog (GtkWindow *parent, const gchar *dsn)
         gtk_widget_show (label);
 
 	/* create data source settings page */
-	props = gnome_db_dsn_config_new ();
-	gnome_db_dsn_config_set_info (GNOME_DB_DSN_CONFIG (props), old_dsn_info);
+	props = gnome_db_dsn_editor_new ();
+	gnome_db_dsn_editor_set_dsn (GNOME_DB_DSN_EDITOR (props), old_dsn_info);
 	gtk_widget_show (props);
 	g_signal_connect (G_OBJECT (props), "changed", G_CALLBACK (dsn_changed_cb), dialog);	
 	gtk_box_pack_start (GTK_BOX (hbox), props, TRUE, TRUE, 0);
@@ -165,21 +168,21 @@ dsn_properties_dialog (GtkWindow *parent, const gchar *dsn)
 		switch (result) {
 		case REVERT_BUTTON:
 			/* reverting changes... */
-			gnome_db_dsn_config_set_info (GNOME_DB_DSN_CONFIG (props), old_dsn_info);
+			gnome_db_dsn_editor_set_dsn (GNOME_DB_DSN_EDITOR (props), old_dsn_info);
 			gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), REVERT_BUTTON, FALSE);
 			break;
 		case TEST_BUTTON:
 		{
-			GtkWidget *dialog, *login;
+			GtkWidget *dialog;
 			gchar *title;
 			GdaConnection *cnc = NULL;
 
 			title = g_strdup_printf (_("Login for %s"), dsn);
 			dialog = gnome_db_login_dialog_new (title);
 			g_free (title);
-			login = gnome_db_login_dialog_get_login_widget (GNOME_DB_LOGIN_DIALOG (dialog));
-			gnome_db_login_set_dsn (GNOME_DB_LOGIN (login), dsn);
-			gnome_db_login_set_show_dsn_selector (GNOME_DB_LOGIN (login), FALSE);
+			GnomeDbLogin* login = gnome_db_login_dialog_get_login_widget (GNOME_DB_LOGIN_DIALOG (dialog));
+			gnome_db_login_set_dsn (login, dsn);
+			gnome_db_login_set_show_dsn_selector (login, FALSE);
 
 			if (gnome_db_login_dialog_run (GNOME_DB_LOGIN_DIALOG (dialog))) {
 				GtkWidget *msgdialog;
@@ -189,9 +192,9 @@ dsn_properties_dialog (GtkWindow *parent, const gchar *dsn)
 					db_client = gda_client_new ();
 				
 				cnc = gda_client_open_connection (db_client,
-								  gnome_db_login_get_dsn (GNOME_DB_LOGIN (login)),
-								  gnome_db_login_get_username (GNOME_DB_LOGIN (login)),
-								  gnome_db_login_get_password (GNOME_DB_LOGIN (login)),
+								  gnome_db_login_get_dsn (login),
+								  gnome_db_login_get_username (login),
+								  gnome_db_login_get_password (login),
 								  0, &error);
 				if (cnc && GDA_IS_CONNECTION (cnc)) {
 					str = g_strdup_printf ("<b>%s</b>", _("Connection successfully opened!"));
@@ -222,16 +225,16 @@ dsn_properties_dialog (GtkWindow *parent, const gchar *dsn)
 		case BROWSE_BUTTON:
 		{
 			char *argv[3];
-			gboolean result;
+			gboolean sresult;
 			
 			/* run gnome-database-properties dictig tool */
 			argv[0] = (char *) "gnome-db-browser";
 			argv[1] = (char *) dsn;
 			argv[2] = NULL;
 			
-			result = g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+			sresult = g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 						NULL, NULL, NULL, NULL);
-			if (!result) {
+			if (!sresult) {
 				GtkWidget *msgdialog;
 				str = g_strdup_printf ("<b>%s</b>", _("Could not execute browser program (gnome-db-browser)."));
 				msgdialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (dialog), GTK_DIALOG_MODAL,
@@ -240,6 +243,8 @@ dsn_properties_dialog (GtkWindow *parent, const gchar *dsn)
 				gtk_dialog_run (GTK_DIALOG (msgdialog));
 				gtk_widget_destroy (msgdialog);
 			}
+			else
+				result = GTK_RESPONSE_OK; /* force closing of this dialog */
 		}
 		break;
 		default:
