@@ -30,7 +30,7 @@
 #include <libgnomedb/gnome-db-util.h>
 #include "dsn-config.h"
 #include "provider-config.h"
-
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 GtkWindow *main_window;
 GtkActionGroup *actions;
@@ -139,8 +139,12 @@ about_cb (GtkAction *action, gpointer user_data)
 	const gchar *translator_credits =
 		"Christian Rose <menthos@menthos.com> Swedish translations\n" \
 		"Kjartan Maraas <kmaraas@online.no> Norwegian translation\n";
+	
+	gchar *file;
 
-	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	file = gnome_db_gbr_get_icon_path ("gnome-db.png");
+	icon = gdk_pixbuf_new_from_file (file, NULL);
+	g_free (file);
 
 	dialog = gtk_about_dialog_new ();
 	gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (dialog), _("Database Access Properties"));
@@ -234,6 +238,7 @@ create_main_window (gpointer user_data)
 	GtkWidget *window, *vbox;
 	GtkWidget *nb;
 	GdkPixbuf *icon;
+	gchar *file;
 
 	/* create the main window */
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -245,7 +250,9 @@ create_main_window (gpointer user_data)
 			  G_CALLBACK (window_closed_cb), NULL);
 
 	/* icon */
-	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	file = gnome_db_gbr_get_icon_path ("gnome-db.png");
+	icon = gdk_pixbuf_new_from_file (file, NULL);
+	g_free (file);
 	if (icon) {
 		gtk_window_set_icon (GTK_WINDOW (window), icon);
 		g_object_unref (icon);
@@ -269,7 +276,12 @@ create_main_window (gpointer user_data)
 int
 main (int argc, char *argv[])
 {
-	bindtextdomain (GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
+	gchar *str;
+
+	str = gnome_db_gbr_get_locale_dir_path ();
+	bindtextdomain (GETTEXT_PACKAGE, str);
+	g_free (str);
+
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
