@@ -26,7 +26,7 @@
 
 static void gnome_db_canvas_field_class_init (GnomeDbCanvasFieldClass * class);
 static void gnome_db_canvas_field_init       (GnomeDbCanvasField * drag);
-static void gnome_db_canvas_field_finalize   (GObject   * object);
+static void gnome_db_canvas_field_dispose   (GObject   * object);
 
 static void gnome_db_canvas_field_set_property (GObject *object,
 					  guint param_id,
@@ -57,7 +57,7 @@ gnome_db_canvas_field_get_type (void)
 {
 	static GType type = 0;
 
-        if (!type) {
+        if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbCanvasFieldClass),
 			(GBaseInitFunc) NULL,
@@ -85,7 +85,7 @@ gnome_db_canvas_field_class_init (GnomeDbCanvasFieldClass * class)
 
 	field_parent_class = g_type_class_peek_parent (class);
 
-	object_class->finalize = gnome_db_canvas_field_finalize;
+	object_class->dispose = gnome_db_canvas_field_dispose;
 	GNOME_DB_CANVAS_ITEM_CLASS (class)->extra_event = gnome_db_canvas_field_extra_event;
 
 	/* Properties */
@@ -109,11 +109,13 @@ field_destroyed_cb (GdaEntityField *field, GnomeDbCanvasField *cf)
 {
 	g_signal_handlers_disconnect_by_func (G_OBJECT (field),
 					      G_CALLBACK (field_destroyed_cb), cf);
+	g_object_unref (cf->priv->field);
+	cf->priv->field = NULL;
 	gtk_object_destroy (GTK_OBJECT (cf));
 }
 
 static void
-gnome_db_canvas_field_finalize (GObject   * object)
+gnome_db_canvas_field_dispose (GObject   * object)
 {
 	GnomeDbCanvasField *cf;
 	g_return_if_fail (object != NULL);
@@ -133,7 +135,7 @@ gnome_db_canvas_field_finalize (GObject   * object)
 	}
 
 	/* for the parent class */
-	field_parent_class->finalize (object);
+	field_parent_class->dispose (object);
 }
 
 static void 
@@ -159,15 +161,12 @@ gnome_db_canvas_field_set_property    (GObject *object,
 		if (cf->priv->field) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (cf->priv->field),
 							      G_CALLBACK (field_destroyed_cb), cf);
-
 			g_object_unref (cf->priv->field);
 			cf->priv->field = NULL;
 		}
 
 		cf->priv->field = GDA_ENTITY_FIELD (propobject);
-		if (cf->priv->field)
-			g_object_ref (cf->priv->field);
-
+		g_object_ref (cf->priv->field);
 		gda_object_connect_destroy (cf->priv->field, G_CALLBACK (field_destroyed_cb), cf);
 		
 		/* attributes setting */
@@ -208,7 +207,6 @@ gnome_db_canvas_field_get_property    (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbCanvasField *cf;
-	gpointer ptr;
 
 	cf = GNOME_DB_CANVAS_FIELD (object);
 
