@@ -1,6 +1,6 @@
 /* gnome-db-entry-shell.c
  *
- * Copyright (C) 2003 - 2004 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,21 +20,23 @@
 
 #include <gdk/gdkkeysyms.h>
 #include "gnome-db-entry-shell.h"
+#include "gnome-db-entry-none.h"
 #include <libgda/gda-data-handler.h>
 #include <libgnomedb/utility.h>
+#include <glib/gi18n-lib.h>
 
-static void gnome_db_entry_shell_class_init (GnomeDbEntryShellClass * class);
-static void gnome_db_entry_shell_init (GnomeDbEntryShell * wid);
-static void gnome_db_entry_shell_dispose (GObject   * object);
+static void gnome_db_entry_shell_class_init (GnomeDbEntryShellClass *class);
+static void gnome_db_entry_shell_init (GnomeDbEntryShell *wid);
+static void gnome_db_entry_shell_dispose (GObject *object);
 
-static void gnome_db_entry_shell_set_property    (GObject *object,
-					    guint param_id,
-					    const GValue *value,
-					    GParamSpec *pspec);
-static void gnome_db_entry_shell_get_property    (GObject *object,
-					    guint param_id,
-					    GValue *value,
-					    GParamSpec *pspec);
+static void gnome_db_entry_shell_set_property (GObject *object,
+					       guint param_id,
+					       const GValue *value,
+					       GParamSpec *pspec);
+static void gnome_db_entry_shell_get_property (GObject *object,
+					       guint param_id,
+					       GValue *value,
+					       GParamSpec *pspec);
 
 
 static gint event_cb (GtkWidget *widget, GdkEvent *event, GnomeDbEntryShell *shell);
@@ -82,7 +84,7 @@ gnome_db_entry_shell_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbEntryShellClass),
 			(GBaseInitFunc) NULL,
@@ -209,9 +211,9 @@ gnome_db_entry_shell_dispose (GObject   * object)
 
 static void
 gnome_db_entry_shell_set_property (GObject *object,
-			     guint param_id,
-			     const GValue *value,
-			     GParamSpec *pspec)
+				   guint param_id,
+				   const GValue *value,
+				   GParamSpec *pspec)
 {
 	gpointer ptr;
 	GnomeDbEntryShell *shell;
@@ -230,6 +232,10 @@ gnome_db_entry_shell_set_property (GObject *object,
 				gda_object_connect_destroy (shell->priv->data_handler, 
 							 G_CALLBACK (handler_destroyed_cb), shell);
 			}
+			else 
+				g_message (_("Widget of class '%s' does not have any associated GdaDataHandler, "
+					     "(to be set using the 'handler' property) expect some mis-behaviours"), 
+					   G_OBJECT_TYPE_NAME (object));
 			break;
 		case PROP_ACTIONS:
 			shell->priv->show_actions = g_value_get_boolean (value);
@@ -264,9 +270,9 @@ handler_destroyed_cb (GdaDataHandler *dh, GnomeDbEntryShell *shell)
 
 static void
 gnome_db_entry_shell_get_property (GObject *object,
-			     guint param_id,
-			     GValue *value,
-			     GParamSpec *pspec)
+				   guint param_id,
+				   GValue *value,
+				   GParamSpec *pspec)
 {
 	GnomeDbEntryShell *shell;
 
@@ -274,6 +280,10 @@ gnome_db_entry_shell_get_property (GObject *object,
 	if (shell->priv) {
 		switch (param_id) {
 		case PROP_HANDLER:
+			if (!shell->priv->data_handler && !GNOME_DB_IS_ENTRY_NONE (object))
+				g_message (_("Widget of class '%s' does not have any associated GdaDataHandler, "
+					     "(to be set using the 'handler' property) expect some mis-behaviours"), 
+					   G_OBJECT_TYPE_NAME (object));
 			g_value_set_pointer (value, shell->priv->data_handler);
 			break;
 		case PROP_ACTIONS:
