@@ -50,8 +50,8 @@ static GValue         *gnome_db_entry_wrapper_get_value         (GnomeDbDataEntr
 static void            gnome_db_entry_wrapper_set_value_orig    (GnomeDbDataEntry *de, const GValue *value);
 static const GValue   *gnome_db_entry_wrapper_get_value_orig    (GnomeDbDataEntry *de);
 static void            gnome_db_entry_wrapper_set_value_default (GnomeDbDataEntry *de, const GValue *value);
-static void            gnome_db_entry_wrapper_set_attributes    (GnomeDbDataEntry *de, guint attrs, guint mask);
-static guint           gnome_db_entry_wrapper_get_attributes    (GnomeDbDataEntry *de);
+static void            gnome_db_entry_wrapper_set_attributes    (GnomeDbDataEntry *de, GdaValueAttribute attrs, guint mask);
+static GdaValueAttribute gnome_db_entry_wrapper_get_attributes    (GnomeDbDataEntry *de);
 static GdaDataHandler *gnome_db_entry_wrapper_get_handler       (GnomeDbDataEntry *de);
 static gboolean        gnome_db_entry_wrapper_expand_in_layout  (GnomeDbDataEntry *de);
 static void            gnome_db_entry_wrapper_set_editable      (GnomeDbDataEntry *de, gboolean editable);
@@ -94,7 +94,7 @@ gnome_db_entry_wrapper_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbEntryWrapperClass),
 			(GBaseInitFunc) NULL,
@@ -647,8 +647,7 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 
 	/* Setting to DEFAULT */
 	if (mask & GDA_VALUE_ATTR_IS_DEFAULT) {
-		if ((mask & GDA_VALUE_ATTR_CAN_BE_DEFAULT) &&
-		    !(attrs & GDA_VALUE_ATTR_CAN_BE_DEFAULT))
+		if (! (mask & GDA_VALUE_ATTR_CAN_BE_DEFAULT))
 			g_return_if_reached ();
 		if (attrs & GDA_VALUE_ATTR_IS_DEFAULT) {
 			block_signals (mgwrap);
@@ -715,10 +714,10 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 	g_signal_emit_by_name (G_OBJECT (mgwrap), "status_changed");
 }
 
-static guint
+static GdaValueAttribute
 gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 {
-	guint retval = 0;
+	GdaValueAttribute retval = 0;
 	GnomeDbEntryWrapper *mgwrap;
 	GValue *value = NULL;
 	gboolean has_current_value;
@@ -728,6 +727,7 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_val_if_fail (mgwrap->priv, 0);
 
+	check_correct_init (mgwrap);
 	if (!mgwrap->priv->real_class->value_is_equal_to || 
 	    !mgwrap->priv->real_class->value_is_null) {
 		value = gnome_db_entry_wrapper_get_value (iface);
@@ -738,7 +738,7 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 	
 	/* NULL? */
 	if (has_current_value) {
-		if (G_VALUE_TYPE (value) == GDA_TYPE_NULL) {
+		if ((value && (G_VALUE_TYPE (value) == GDA_TYPE_NULL)) || !value) {
 			if (mgwrap->priv->default_forced) {
 				if (mgwrap->priv->null_forced)
 					value_is_null = TRUE;
