@@ -105,7 +105,7 @@ gnome_db_basic_form_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbBasicFormClass),
 			(GBaseInitFunc) NULL,
@@ -138,15 +138,15 @@ gnome_db_basic_form_class_init (GnomeDbBasicFormClass * class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GnomeDbBasicFormClass, param_changed),
 			      NULL, NULL,
-			      marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2,
-			      G_TYPE_OBJECT, G_TYPE_BOOLEAN);
+			      gnome_db_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2,
+			      GDA_TYPE_PARAMETER, G_TYPE_BOOLEAN);
 	gnome_db_basic_form_signals[ACTIVATED] =
 		g_signal_new ("activated",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GnomeDbBasicFormClass, activated),
 			      NULL, NULL,
-			      marshal_VOID__VOID, G_TYPE_NONE, 0);
+			      gnome_db_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->param_changed = NULL;
 	class->activated = NULL;
@@ -247,14 +247,15 @@ gnome_db_basic_form_new_custom (GdaParameterList *paramlist, const gchar *glade_
 {
 	GnomeDbFormLayoutSpec spec;
 	GObject *obj;
-
+#ifdef HAVE_LIBGLADE
 	spec.xml_object = NULL;
+#endif
 	spec.xml_file = (gchar *) glade_file;
 	spec.root_element = (gchar *) root_element;
 	spec.form_prefix = (gchar *) form_prefix;
 	obj = g_object_new (GNOME_DB_TYPE_BASIC_FORM, "layout_spec", &spec, "paramlist", paramlist, NULL);
 
-	return (GtkWidget *) obj;	
+	return (GtkWidget *) obj;
 }
 
 
@@ -391,12 +392,15 @@ gnome_db_basic_form_set_property (GObject *object,
 				  GParamSpec *pspec)
 {
 	GnomeDbBasicForm *form;
+#ifdef HAVE_LIBGLADE
 	GnomeDbFormLayoutSpec *lspec, *new_spec = NULL;
+#endif
 
         form = GNOME_DB_BASIC_FORM (object);
         if (form->priv) {
                 switch (param_id) {
 		case PROP_LAYOUT_SPEC:
+#ifdef HAVE_LIBGLADE
 			lspec = g_value_get_pointer (value);
 			if (lspec) {
 				g_return_if_fail (lspec->xml_file || lspec->xml_object);
@@ -432,11 +436,14 @@ gnome_db_basic_form_set_property (GObject *object,
 				g_print ("Loaded Glade file, reinit interface\n");
 			}
 			gnome_db_basic_form_fill (form);
+#else
+			g_warning (_("Libglade support not built."));
+#endif
 			break;
 		case PROP_PARAMLIST:
 			if (form->priv->paramlist) {
-				TO_IMPLEMENT;
-				g_assert_not_reached ();
+				paramlist_destroyed_cb (form->priv->paramlist, form);
+				gnome_db_basic_form_clean (form);
 			}
 
 			form->priv->paramlist = g_value_get_pointer (value);
@@ -557,8 +564,10 @@ gnome_db_basic_form_clean (GnomeDbBasicForm *form)
 static void
 layout_spec_free (GnomeDbFormLayoutSpec *spec)
 {
+#ifdef HAVE_LIBGLADE
 	if (spec->xml_object)
 		g_object_unref (spec->xml_object);
+#endif
 	g_free (spec->xml_file);
 	g_free (spec->root_element);
 	g_free (spec->form_prefix);
@@ -623,6 +632,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			/* create entry */
 			g_object_get (G_OBJECT (param), "entry_plugin", &plugin, NULL);
 			entry = GTK_WIDGET (gnome_db_util_new_data_entry (dict, type, plugin));
+			g_free(plugin);
 
 
 			/* set current value */
@@ -697,6 +707,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	/*
 	 * If there is a layout spec, then try to use it
 	 */
+#ifdef HAVE_LIBGLADE
 	if (form->priv->layout_spec) {
 		GtkWidget *layout = NULL;
 		
@@ -743,6 +754,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			gtk_widget_show (layout);
 		}
 	}
+#endif
 
 	/* 
 	 * There is no layout spec (or the provided one could not be used),
@@ -862,9 +874,26 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	g_object_set_data (G_OBJECT (form), "expand", GINT_TO_POINTER (form_expand));
 	
 	/* Set the Show actions in the entries */
-	gnome_db_basic_form_show_entry_actions(form, form->priv->show_actions);
+	gnome_db_basic_form_show_entry_actions (form, form->priv->show_actions);
 	/* Set the Auto entries default in the entries */
-	gnome_db_basic_form_set_entries_auto_default(form, form->priv->entries_auto_default);
+	gnome_db_basic_form_set_entries_auto_default (form, form->priv->entries_auto_default);
+
+	/* hide some entries if specified */
+	for (list = form->priv->entries; list; list = list->next) {
+		gboolean explicit_hide = FALSE;
+		GdaParameter *param;
+			
+		param = g_object_get_data (G_OBJECT (list->data), "param");
+		if (param) {
+			if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "_gnome_db_explicit_show")) == 1)
+				explicit_hide = TRUE;
+		}
+		else {
+			/* FIXME */
+		}
+		if (explicit_hide) 
+			gnome_db_basic_form_entry_show (form, param, FALSE);
+	}
 }
 
 static void
@@ -910,7 +939,7 @@ mark_not_null_entry_labels (GnomeDbBasicForm *form, gboolean show_mark)
 		gchar *str;
 
 		title = g_object_get_data (G_OBJECT (list->data), "_gda_title");
-		str = utility_markup_title (title, !show_mark);
+		str = gnome_db_utility_markup_title (title, !show_mark);
 		if (show_mark) 
 			gtk_label_set_markup (GTK_LABEL (list->data), str);
 		else 
@@ -935,7 +964,7 @@ static void
 entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 {
 	GdaParameter *param;
-	guint attr;
+	GdaValueAttribute attr;
 
 	attr = gnome_db_data_entry_get_attributes (entry);
 	param = g_object_get_data (G_OBJECT (entry), "param");
