@@ -1,6 +1,6 @@
 /* gnome-db-data-widget.h
  *
- * Copyright (C) 2004 - 2006 Vivien Malerba
+ * Copyright (C) 2004 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,7 +35,12 @@ G_BEGIN_DECLS
 #define GNOME_DB_IS_DATA_WIDGET(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, GNOME_DB_TYPE_DATA_WIDGET)
 #define GNOME_DB_DATA_WIDGET_GET_IFACE(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GNOME_DB_TYPE_DATA_WIDGET, GnomeDbDataWidgetIface))
 
-
+typedef enum {
+	GNOME_DB_DATA_WIDGET_WRITE_ON_DEMAND           = 0, /* write only when explicitely requested */
+	GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE       = 1, /* write when the current selected row changes */
+	GNOME_DB_DATA_WIDGET_WRITE_ON_VALUE_ACTIVATED  = 2, /* write when user activates a value change */
+	GNOME_DB_DATA_WIDGET_WRITE_ON_VALUE_CHANGE     = 3  /* write when a parameters's value changes */
+} GnomeDbDataWidgetWriteMode;
 
 /* struct for the interface */
 struct _GnomeDbDataWidgetIface
@@ -52,9 +57,12 @@ struct _GnomeDbDataWidgetIface
 
 	GdaDataModel        *(* get_gda_model)       (GnomeDbDataWidget *iface);
 	void                 (* set_gda_model)       (GnomeDbDataWidget *iface, GdaDataModel *model);
+	gboolean             (* set_write_mode)      (GnomeDbDataWidget *iface, GnomeDbDataWidgetWriteMode mode);
+	GnomeDbDataWidgetWriteMode (* get_write_mode)(GnomeDbDataWidget *iface);
 
 	/* signals */
 	void                 (* proxy_changed)       (GnomeDbDataWidget *iface, GdaDataProxy *proxy);
+	void                 (* iter_changed)        (GnomeDbDataWidget *iface, GdaDataModelIter *iter);
 };
 
 GType             gnome_db_data_widget_get_type                  (void) G_GNUC_CONST;
@@ -73,6 +81,8 @@ void              gnome_db_data_widget_set_gda_model             (GnomeDbDataWid
 								  GdaDataModel *model);
 GdaDataModelIter *gnome_db_data_widget_get_current_data          (GnomeDbDataWidget *iface);
 
+gboolean          gnome_db_data_widget_set_write_mode            (GnomeDbDataWidget *iface, GnomeDbDataWidgetWriteMode mode);
+GnomeDbDataWidgetWriteMode gnome_db_data_widget_get_write_mode   (GnomeDbDataWidget *iface);
 
 G_END_DECLS
 
