@@ -369,7 +369,7 @@ forward_page_function (gint current_page, GnomeDbDsnAssistant *assistant)
 
 static void
 gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
-				    GnomeDbDsnAssistantClass *klass)
+			     GnomeDbDsnAssistantClass *klass)
 {
 	GtkWidget *label, *vbox, *table;
 	GtkAssistant *assist;
@@ -379,6 +379,7 @@ gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
 
 	/* global assistant settings */
 	assist = GTK_ASSISTANT (assistant);
+	gtk_window_set_title (GTK_WINDOW (assist), _("New data source definition"));
 	g_signal_connect (assist, "cancel", G_CALLBACK (assistant_cancelled_cb), NULL);
 	g_signal_connect (assist, "apply", G_CALLBACK (assistant_applied_cb), NULL);
 	gtk_assistant_set_forward_page_func (assist, (GtkAssistantPageFunc) forward_page_function, 
@@ -391,7 +392,7 @@ gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
 	assistant->priv->create_db_op = NULL;
 
 	/* load icons */
-	str = gnome_db_gbr_get_icon_path ("gnome-db.png");
+	str = gnome_db_gbr_get_file_path (GNOME_DB_DATA_DIR, "pixmaps", "libgnomedb-" ABI_VERSION, "gnome-db.png", NULL);
 	assistant->priv->logo = gdk_pixbuf_new_from_file (str, NULL);
 	g_free (str);
 
@@ -628,7 +629,8 @@ gnome_db_dsn_assistant_finalize (GObject *object)
 	if (assistant->priv->client)
 		g_object_unref (assistant->priv->client);
 
-	gdk_pixbuf_unref (assistant->priv->logo);
+	if (assistant->priv->logo)
+		gdk_pixbuf_unref (assistant->priv->logo);
 	if (assistant->priv->dsn_info)
 		gda_data_source_info_free (assistant->priv->dsn_info);
 
