@@ -53,6 +53,7 @@ enum
         PROP_0,
         PROP_RAW_GRID,
 	PROP_INFO,
+	PROP_MODEL
 };
 
 GType
@@ -60,7 +61,7 @@ gnome_db_grid_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbGridClass),
 			(GBaseInitFunc) NULL,
@@ -97,6 +98,10 @@ gnome_db_grid_class_init (GnomeDbGridClass *class)
                                          g_param_spec_object ("widget_info", NULL, NULL, 
 							      GNOME_DB_TYPE_DATA_WIDGET_INFO,
 							      G_PARAM_READABLE));
+	g_object_class_install_property (object_class, PROP_MODEL,
+	                                 g_param_spec_object ("model", NULL, NULL,
+	                                                      GDA_TYPE_DATA_MODEL,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -120,7 +125,7 @@ gnome_db_grid_init (GnomeDbGrid *grid)
 
 	grid->priv->info = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (grid->priv->raw_grid), 
 							  GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW |
-							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS |
+							  GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS |
 							  GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS);
 	gtk_box_pack_start (GTK_BOX (grid), grid->priv->info, FALSE, TRUE, 0);
 	gtk_widget_show (grid->priv->info);
@@ -141,9 +146,8 @@ gnome_db_grid_new (GdaDataModel *model)
 	
 	g_return_val_if_fail (!model || GDA_IS_DATA_MODEL (model), NULL);
 
-	grid = (GnomeDbGrid *) g_object_new (GNOME_DB_TYPE_GRID, NULL);
-
-	g_object_set(grid->priv->raw_grid, "model", model, NULL);
+	grid = (GnomeDbGrid *) g_object_new (GNOME_DB_TYPE_GRID,
+	                                     "model", model, NULL);
 
 	return (GtkWidget *) grid;
 }
@@ -156,11 +160,16 @@ gnome_db_grid_set_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbGrid *grid;
+	GdaDataModel *model;
 	
 	grid = GNOME_DB_GRID (object);
 	
 	switch (param_id) {
-		
+	case PROP_MODEL:
+		model = GDA_DATA_MODEL (g_value_get_object (value));
+		g_object_set(G_OBJECT (grid->priv->raw_grid),
+		             "model", model, NULL);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -174,6 +183,7 @@ gnome_db_grid_get_property (GObject *object,
 				 GParamSpec *pspec)
 {
 	GnomeDbGrid *grid;
+	GdaDataModel *model;
 
 	grid = GNOME_DB_GRID (object);
 	
@@ -184,6 +194,11 @@ gnome_db_grid_get_property (GObject *object,
 	case PROP_INFO:
 		g_value_set_object (value, grid->priv->info);
 		break;
+	case PROP_MODEL:
+		g_object_get (G_OBJECT (grid->priv->raw_grid),
+		              "model", &model, NULL);
+		g_value_take_object (value, G_OBJECT (model));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
