@@ -283,21 +283,43 @@ gnome_db_login_get_type (void)
 }
 
 #ifdef HAVE_GTKTWOTEN
+static gboolean
+idle_assistant_destroy_cb (GtkWidget *assist)
+{
+	gtk_widget_destroy (assist);
+	return FALSE;
+}
+
 static void
 assistant_finished_cb (GnomeDbDsnAssistant *assistant, gboolean error, GnomeDbLogin *login)
 {
-	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (assistant)));
-	gtk_widget_set_sensitive (gtk_widget_get_toplevel (GTK_WIDGET (login)), TRUE);
+	GtkWidget *toplevel;
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (login));
+	g_idle_add_full (G_PRIORITY_HIGH_IDLE, (GSourceFunc) idle_assistant_destroy_cb, 
+			 gtk_widget_get_toplevel (GTK_WIDGET (assistant)), NULL);
+	gtk_widget_set_sensitive (toplevel, TRUE);
+	gtk_window_set_modal (GTK_WINDOW (toplevel), 
+			      GPOINTER_TO_INT (g_object_get_data (G_OBJECT (toplevel), "__modal")));
 }
 
 static void
 add_dsn_cb (GtkButton *button, GnomeDbLogin *login)
 {
 	GtkWidget *assistant;
+	GtkWidget *toplevel;
+	gboolean is_modal;
 
-	gtk_widget_set_sensitive (gtk_widget_get_toplevel (GTK_WIDGET (login)), FALSE);
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (login));
+	gtk_widget_set_sensitive (toplevel, FALSE);
+	is_modal = gtk_window_get_modal (GTK_WINDOW (toplevel));
+	g_object_set_data (G_OBJECT (toplevel), "__modal", GINT_TO_POINTER (is_modal));
+	gtk_window_set_modal (GTK_WINDOW (toplevel), FALSE);
 
 	assistant = gnome_db_dsn_assistant_new ();
+	gtk_window_set_modal (GTK_WINDOW (assistant), is_modal);
+	gtk_window_set_transient_for (GTK_WINDOW (assistant), GTK_WINDOW (toplevel));
+
 	g_signal_connect (G_OBJECT (assistant), "finished",
 			  G_CALLBACK (assistant_finished_cb), login);
 	gtk_widget_show (assistant);
