@@ -19,12 +19,14 @@
  */
 
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include <libgda/libgda.h>
 #include <libgda/gda-data-proxy.h>
 #include <glib/gi18n-lib.h>
 #include "gnome-db-raw-form.h"
 #include "gnome-db-data-widget.h"
 #include "gnome-db-basic-form.h"
+#include "gnome-db-data-widget-filter.h"
 #include "utility.h"
 
 static void gnome_db_raw_form_class_init (GnomeDbRawFormClass * class);
@@ -70,6 +72,9 @@ struct _GnomeDbRawFormPriv
 	GnomeDbDataWidgetWriteMode  write_mode;
 
 	GtkActionGroup             *actions_group;
+
+	GtkWidget                  *filter;
+	GtkWidget                  *filter_window;
 };
 
 #define PAGE_NO_DATA 0
@@ -156,6 +161,7 @@ static void action_first_record_cb (GtkAction *action, GnomeDbRawForm *form);
 static void action_prev_record_cb (GtkAction *action, GnomeDbRawForm *form);
 static void action_next_record_cb (GtkAction *action, GnomeDbRawForm *form);
 static void action_last_record_cb (GtkAction *action, GnomeDbRawForm *form);
+static void action_filter_cb (GtkAction *action, GnomeDbRawForm *form);
 
 static GtkActionEntry ui_actions[] = {
 	{ "ActionNew", GTK_STOCK_ADD, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
@@ -171,7 +177,9 @@ static GtkActionEntry ui_actions[] = {
 	{ "ActionPrevRecord", GTK_STOCK_GO_BACK, "_Previous record", NULL, "Go to previous record of records", 
 	  G_CALLBACK (action_prev_record_cb)},
 	{ "ActionNextRecord", GTK_STOCK_GO_FORWARD, "Ne_xt record", NULL, "Go to next record of records",
-	  G_CALLBACK (action_next_record_cb)}
+	  G_CALLBACK (action_next_record_cb)},
+	{ "ActionFilter", GTK_STOCK_FIND, "Filter", NULL, "Filter records",
+	  G_CALLBACK (action_filter_cb)}
 };
 
 static void
@@ -251,6 +259,9 @@ gnome_db_raw_form_init (GnomeDbRawForm *wid)
 	action = gtk_action_group_get_action (wid->priv->actions_group, "ActionNew");
 	g_signal_connect (G_OBJECT (action), "activate",
 			  G_CALLBACK (action_new_activated_cb), wid);
+
+	wid->priv->filter = NULL;
+	wid->priv->filter_window = NULL;
 }
 
 /**
@@ -281,6 +292,11 @@ gnome_db_raw_form_dispose (GObject *object)
 	form = GNOME_DB_RAW_FORM (object);
 
 	if (form->priv) {
+		if (form->priv->filter) 
+			gtk_widget_destroy (form->priv->filter);
+		if (form->priv->filter_window) 
+			gtk_widget_destroy (form->priv->filter_window);
+
 		/* proxy's iterator */
 		if (form->priv->iter) {
 			g_signal_handlers_disconnect_by_func (form->priv->iter,
@@ -706,6 +722,155 @@ action_last_record_cb (GtkAction *action, GnomeDbRawForm *form)
 	arrow_actions_real_do (form, 2);
 }
 
+static void
+hide_filter_window (GnomeDbRawForm *form)
+{
+	gtk_widget_hide (form->priv->filter_window);
+	gtk_grab_remove (form->priv->filter_window);
+}
+
+static gboolean
+filter_event (GtkWidget *widget, GdkEventAny *event, GnomeDbRawForm *form)
+{
+	hide_filter_window (form);
+	return TRUE;
+}
+
+static gboolean
+key_press_filter_event (GtkWidget *widget, GdkEventKey *event, GnomeDbRawForm *form)
+{
+	if (event->keyval == GDK_Escape ||
+	    event->keyval == GDK_Tab ||
+            event->keyval == GDK_KP_Tab ||
+            event->keyval == GDK_ISO_Left_Tab) {
+		hide_filter_window (form);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+filter_position_func (GtkWidget *widget,
+		      GtkWidget *search_dialog,
+		      gpointer   user_data)
+{
+	gint x, y;
+	gint tree_x, tree_y;
+	gint tree_width, tree_height;
+	GdkWindow *window = GTK_WIDGET (widget)->window;
+	GdkScreen *screen = gdk_drawable_get_screen (window);
+	GtkRequisition requisition;
+	gint monitor_num;
+	GdkRectangle monitor;
+
+	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+  
+	gtk_widget_realize (search_dialog);
+
+	gdk_window_get_origin (window, &tree_x, &tree_y);
+	gdk_drawable_get_size (window,
+			       &tree_width,
+			       &tree_height);
+	gtk_widget_size_request (search_dialog, &requisition);
+
+	if (tree_x + tree_width > gdk_screen_get_width (screen))
+		x = gdk_screen_get_width (screen) - requisition.width;
+	else if (tree_x + tree_width - requisition.width < 0)
+		x = 0;
+	else
+		x = tree_x + tree_width - requisition.width;
+
+	if (tree_y + tree_height + requisition.height > gdk_screen_get_height (screen))
+		y = gdk_screen_get_height (screen) - requisition.height;
+	else if (tree_y + tree_height < 0) /* isn't really possible ... */
+		y = 0;
+	else
+		y = tree_y + tree_height;
+
+	gtk_window_move (GTK_WINDOW (search_dialog), x, y);
+}
+
+static gboolean
+popup_grab_on_window (GdkWindow  *window,
+                      guint32     activate_time) 
+{
+        if (gdk_pointer_grab (window, TRUE,
+                              GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
+                              NULL, NULL,
+                              activate_time) == 0) {
+
+                if (gdk_keyboard_grab (window, TRUE, activate_time) == 0) 
+                        return TRUE;
+		else {
+                        gdk_pointer_ungrab (activate_time);
+                        return FALSE;
+                }
+        }
+
+        return FALSE;
+}
+
+static void
+action_filter_cb (GtkAction *action, GnomeDbRawForm *form)
+{
+	GtkWidget *toplevel;
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (form));
+	
+	if (!form->priv->filter_window) {
+		/* create filter window */
+		GtkWidget *frame, *vbox;
+	       
+		form->priv->filter_window = gtk_window_new (GTK_WINDOW_POPUP);
+
+		gtk_widget_set_events (form->priv->filter_window,
+				       gtk_widget_get_events (form->priv->filter_window) | GDK_KEY_PRESS_MASK);
+
+		if (GTK_WIDGET_TOPLEVEL (toplevel) && GTK_WINDOW (toplevel)->group)
+			gtk_window_group_add_window (GTK_WINDOW (toplevel)->group,
+						     GTK_WINDOW (form->priv->filter_window));
+		
+		g_signal_connect (form->priv->filter_window, "delete_event",
+				  G_CALLBACK (filter_event), form);
+		g_signal_connect (form->priv->filter_window, "button_press_event",
+				  G_CALLBACK (filter_event), form);
+		g_signal_connect (form->priv->filter_window, "key_press_event",
+				  G_CALLBACK (key_press_filter_event), form);
+		frame = gtk_frame_new (NULL);
+		gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+		gtk_widget_show (frame);
+		gtk_container_add (GTK_CONTAINER (form->priv->filter_window), frame);
+		
+		vbox = gtk_vbox_new (FALSE, 0);
+		gtk_widget_show (vbox);
+		gtk_container_add (GTK_CONTAINER (frame), vbox);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
+		
+		/* add real filter widget */
+		if (! form->priv->filter) {
+			form->priv->filter = gnome_db_data_widget_filter_new (GNOME_DB_DATA_WIDGET (form));
+			gtk_widget_show (form->priv->filter);
+		}
+		gtk_container_add (GTK_CONTAINER (vbox), form->priv->filter);
+	}
+	else if (GTK_WIDGET_TOPLEVEL (toplevel)) {
+		if (GTK_WINDOW (toplevel)->group)
+			gtk_window_group_add_window (GTK_WINDOW (toplevel)->group,
+						     GTK_WINDOW (form->priv->filter_window));
+		else if (GTK_WINDOW (form->priv->filter_window)->group)
+			gtk_window_group_remove_window (GTK_WINDOW (form->priv->filter_window)->group,
+							GTK_WINDOW (form->priv->filter_window));
+	}
+
+	/* move the filter window to a correct location */
+	/* FIXME: let the user specify the position function like GtkTreeView -> search_position_func() */
+	gtk_grab_add (form->priv->filter_window);
+	filter_position_func (GTK_WIDGET (form), form->priv->filter_window, NULL);
+	gtk_widget_show (form->priv->filter_window);
+	popup_grab_on_window (form->priv->filter_window->window,
+			      gtk_get_current_event_time ());
+}
+
 
 static void
 arrow_actions_real_do (GnomeDbRawForm *form, gint movement)
