@@ -24,6 +24,7 @@
 #include <libgda/libgda.h>
 #include "gnome-db-raw-grid.h"
 #include "gnome-db-data-widget.h"
+#include "gnome-db-data-widget-filter.h"
 #include "utility.h"
 #include "gnome-db-tools.h"
 #include "gnome-db-util.h"
@@ -97,6 +98,9 @@ struct _GnomeDbRawGridPriv
 
 	gint                        export_type; /* used by the export dialog */
 	GnomeDbDataWidgetWriteMode  write_mode;
+
+	GtkWidget                  *filter;
+	GtkWidget                  *filter_window;
 };
 
 /* get a pointer to the parents to be able to call their destructor */
@@ -146,6 +150,7 @@ static void action_first_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
 static void action_prev_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
 static void action_next_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
 static void action_last_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid);
+static void action_filter_cb (GtkAction *action, GnomeDbRawGrid *grid);
 
 static GtkActionEntry ui_actions[] = {
 	{ "ActionNew", GTK_STOCK_ADD, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
@@ -161,7 +166,9 @@ static GtkActionEntry ui_actions[] = {
 	{ "ActionPrevChunck", GTK_STOCK_GO_BACK, "_Previous chunck", NULL, "Go to previous chunck of records", 
 	  G_CALLBACK (action_prev_chunck_cb)},
 	{ "ActionNextChunck", GTK_STOCK_GO_FORWARD, "Ne_xt chunck", NULL, "Go to next chunck of records",
-	  G_CALLBACK (action_next_chunck_cb)}
+	  G_CALLBACK (action_next_chunck_cb)},
+	{ "ActionFilter", GTK_STOCK_FIND, "Filter", NULL, "Filter records",
+	  G_CALLBACK (action_filter_cb)}
 };
 
 GType
@@ -211,10 +218,9 @@ gnome_db_raw_grid_widget_init (GnomeDbDataWidgetIface *iface)
 }
 
 static void
-gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
+gnome_db_raw_grid_class_init (GnomeDbRawGridClass *class)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
-	
+	GObjectClass  *object_class = G_OBJECT_CLASS (class);
 	parent_class = g_type_class_peek_parent (class);
 
 	gnome_db_raw_grid_signals[SELECTION_CHANGED] = 
@@ -281,6 +287,7 @@ gnome_db_raw_grid_init (GnomeDbRawGrid *grid)
 			  G_CALLBACK (tree_view_event_cb), grid);
 	g_signal_connect (G_OBJECT (tree_view), "button_press_event",
                           G_CALLBACK (tree_view_popup_button_pressed_cb), grid);
+	gtk_tree_view_set_enable_search (tree_view, FALSE);
 
 	/* selection and signal handling */
 	selection = gtk_tree_view_get_selection (tree_view);
@@ -293,6 +300,9 @@ gnome_db_raw_grid_init (GnomeDbRawGrid *grid)
 	/* action group */
 	grid->priv->actions_group = gtk_action_group_new ("Actions");
 	gtk_action_group_add_actions (grid->priv->actions_group, ui_actions, G_N_ELEMENTS (ui_actions), grid);
+
+	grid->priv->filter = NULL;
+	grid->priv->filter_window = NULL;
 }
 
 /**
@@ -333,6 +343,11 @@ gnome_db_raw_grid_dispose (GObject *object)
 			grid->priv->actions_group = NULL;
 		}
 
+		if (grid->priv->filter) 
+			gtk_widget_destroy (grid->priv->filter);
+		if (grid->priv->filter_window) 
+			gtk_widget_destroy (grid->priv->filter_window);
+
 		/* the private area itself */
 		g_free (grid->priv);
 		grid->priv = NULL;
@@ -529,6 +544,7 @@ static void     data_cell_values_changed (GtkCellRenderer *renderer, const gchar
 					  GSList *new_values, GSList *all_new_values, GnomeDbRawGrid *grid);
 static void     data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, 
 					  GdaValueAttribute requested_action, GnomeDbRawGrid *grid);
+static void     treeview_column_clicked_cb (GtkTreeViewColumn *tree_column, GnomeDbRawGrid *grid);
 
 /*
  * Creates the GtkTreeView's columns, from the grid->priv->store GtkTreeModel
@@ -675,9 +691,10 @@ init_tree_view (GnomeDbRawGrid *grid)
 		g_object_set_data (G_OBJECT (renderer), "group", group);
 
 		/* Sorting data */
-		/* gtk_tree_view_column_set_sort_column_id (column, i); */
-		/* gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (tree_model), */
-		/* i, (GtkTreeIterCompareFunc) tree_sortable_sort_values, node, NULL); */
+		gtk_tree_view_column_set_clickable (column, TRUE);
+		g_signal_connect (G_OBJECT (column), "clicked",
+				  G_CALLBACK (treeview_column_clicked_cb), grid);
+
 		list = g_slist_next (list);
 		i++;
 	}
@@ -980,6 +997,43 @@ set_iter_from_path (GnomeDbRawGrid *grid, const gchar *path, GtkTreeIter *iter)
 	return TRUE;
 }
 
+static void
+treeview_column_clicked_cb (GtkTreeViewColumn *tree_column, GnomeDbRawGrid *grid)
+{
+	/* 
+	const gchar *filter;
+	gchar *newfilter = NULL;
+	GdaParameterListGroup *group;
+
+	group = g_object_get_data (G_OBJECT (tree_column), "group");
+	g_assert (group);
+	filter = gda_data_proxy_get_filter_expr (grid->priv->proxy);
+	if (filter) {
+		TO_IMPLEMENT;
+	}
+	else {
+		GString *string;
+		GSList *nodes;
+
+		string = g_string_new ("ORDER BY ");
+		for (nodes = group->nodes; nodes; nodes = nodes->next) {
+			GdaParameter *param = ((GdaParameterListNode*) nodes->data)->param;
+			g_assert (param);
+			if (nodes != group->nodes)
+				g_string_append (string, ", ");
+			g_string_append (string, gda_object_get_name (GDA_OBJECT (param)));
+		}
+		newfilter = string->str;
+		g_string_free (string, FALSE);
+	}
+	
+	if (newfilter) {
+		g_print ("FILTER: %s\n", newfilter);
+		g_free (newfilter);
+	}
+	*/
+	TO_IMPLEMENT;
+}
 
 /*
  * Actions
@@ -1211,6 +1265,155 @@ action_last_chunck_cb (GtkAction *action, GnomeDbRawGrid *grid)
 	gda_data_proxy_set_sample_start (grid->priv->proxy, G_MAXINT);
 }
 
+static void
+hide_filter_window (GnomeDbRawGrid *grid)
+{
+	gtk_widget_hide (grid->priv->filter_window);
+	gtk_grab_remove (grid->priv->filter_window);
+}
+
+static gboolean
+filter_event (GtkWidget *widget, GdkEventAny *event, GnomeDbRawGrid *grid)
+{
+	hide_filter_window (grid);
+	return TRUE;
+}
+
+static gboolean
+key_press_filter_event (GtkWidget *widget, GdkEventKey *event, GnomeDbRawGrid *grid)
+{
+	if (event->keyval == GDK_Escape ||
+	    event->keyval == GDK_Tab ||
+            event->keyval == GDK_KP_Tab ||
+            event->keyval == GDK_ISO_Left_Tab) {
+		hide_filter_window (grid);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+filter_position_func (GtkWidget *widget,
+		      GtkWidget *search_dialog,
+		      gpointer   user_data)
+{
+	gint x, y;
+	gint tree_x, tree_y;
+	gint tree_width, tree_height;
+	GdkWindow *window = GTK_WIDGET (widget)->window;
+	GdkScreen *screen = gdk_drawable_get_screen (window);
+	GtkRequisition requisition;
+	gint monitor_num;
+	GdkRectangle monitor;
+
+	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+  
+	gtk_widget_realize (search_dialog);
+
+	gdk_window_get_origin (window, &tree_x, &tree_y);
+	gdk_drawable_get_size (window,
+			       &tree_width,
+			       &tree_height);
+	gtk_widget_size_request (search_dialog, &requisition);
+
+	if (tree_x + tree_width > gdk_screen_get_width (screen))
+		x = gdk_screen_get_width (screen) - requisition.width;
+	else if (tree_x + tree_width - requisition.width < 0)
+		x = 0;
+	else
+		x = tree_x + tree_width - requisition.width;
+
+	if (tree_y + tree_height + requisition.height > gdk_screen_get_height (screen))
+		y = gdk_screen_get_height (screen) - requisition.height;
+	else if (tree_y + tree_height < 0) /* isn't really possible ... */
+		y = 0;
+	else
+		y = tree_y + tree_height;
+
+	gtk_window_move (GTK_WINDOW (search_dialog), x, y);
+}
+
+static gboolean
+popup_grab_on_window (GdkWindow  *window,
+                      guint32     activate_time) 
+{
+        if (gdk_pointer_grab (window, TRUE,
+                              GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
+                              NULL, NULL,
+                              activate_time) == 0) {
+
+                if (gdk_keyboard_grab (window, TRUE, activate_time) == 0) 
+                        return TRUE;
+		else {
+                        gdk_pointer_ungrab (activate_time);
+                        return FALSE;
+                }
+        }
+
+        return FALSE;
+}
+
+static void
+action_filter_cb (GtkAction *action, GnomeDbRawGrid *grid)
+{
+	GtkWidget *toplevel;
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (grid));
+	
+	if (!grid->priv->filter_window) {
+		/* create filter window */
+		GtkWidget *frame, *vbox;
+	       
+		grid->priv->filter_window = gtk_window_new (GTK_WINDOW_POPUP);
+
+		gtk_widget_set_events (grid->priv->filter_window,
+				       gtk_widget_get_events (grid->priv->filter_window) | GDK_KEY_PRESS_MASK);
+
+		if (GTK_WIDGET_TOPLEVEL (toplevel) && GTK_WINDOW (toplevel)->group)
+			gtk_window_group_add_window (GTK_WINDOW (toplevel)->group,
+						     GTK_WINDOW (grid->priv->filter_window));
+		
+		g_signal_connect (grid->priv->filter_window, "delete_event",
+				  G_CALLBACK (filter_event), grid);
+		g_signal_connect (grid->priv->filter_window, "button_press_event",
+				  G_CALLBACK (filter_event), grid);
+		g_signal_connect (grid->priv->filter_window, "key_press_event",
+				  G_CALLBACK (key_press_filter_event), grid);
+		frame = gtk_frame_new (NULL);
+		gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+		gtk_widget_show (frame);
+		gtk_container_add (GTK_CONTAINER (grid->priv->filter_window), frame);
+		
+		vbox = gtk_vbox_new (FALSE, 0);
+		gtk_widget_show (vbox);
+		gtk_container_add (GTK_CONTAINER (frame), vbox);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
+		
+		/* add real filter widget */
+		if (! grid->priv->filter) {
+			grid->priv->filter = gnome_db_data_widget_filter_new (GNOME_DB_DATA_WIDGET (grid));
+			gtk_widget_show (grid->priv->filter);
+		}
+		gtk_container_add (GTK_CONTAINER (vbox), grid->priv->filter);
+	}
+	else if (GTK_WIDGET_TOPLEVEL (toplevel)) {
+		if (GTK_WINDOW (toplevel)->group)
+			gtk_window_group_add_window (GTK_WINDOW (toplevel)->group,
+						     GTK_WINDOW (grid->priv->filter_window));
+		else if (GTK_WINDOW (grid->priv->filter_window)->group)
+			gtk_window_group_remove_window (GTK_WINDOW (grid->priv->filter_window)->group,
+							GTK_WINDOW (grid->priv->filter_window));
+	}
+
+	/* move the filter window to a correct location */
+	/* FIXME: let the user specify the position function like GtkTreeView -> search_position_func() */
+	gtk_grab_add (grid->priv->filter_window);
+	filter_position_func (GTK_TREE_VIEW (grid), grid->priv->filter_window, NULL);
+	gtk_widget_show (grid->priv->filter_window);
+	popup_grab_on_window (grid->priv->filter_window->window,
+			      gtk_get_current_event_time ());	
+}
+
 /*
  * Catch any event in the GtkTreeView widget
  */
@@ -1291,6 +1494,8 @@ static void menu_select_all_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 static void menu_unselect_all_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 static void menu_show_columns_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 static void menu_save_as_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
+static void menu_set_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
+static void menu_unset_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid);
 
 static gint
 tree_view_popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, GnomeDbRawGrid *grid)
@@ -1323,6 +1528,13 @@ tree_view_popup_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, Gno
 						       gtk_tree_view_get_headers_visible (tree_view),
 						       G_CALLBACK (menu_show_columns_cb), grid));
 
+	gtk_menu_append (GTK_MENU (menu),
+				 gnome_db_new_menu_item (_("_Set filter"), FALSE,
+							 G_CALLBACK (menu_set_filter_cb), grid));
+	gtk_menu_append (GTK_MENU (menu),
+				 gnome_db_new_menu_item (_("_Unset filter"), FALSE,
+							 G_CALLBACK (menu_unset_filter_cb), grid));
+
 	if (sel_mode != GTK_SELECTION_NONE) {
 		gtk_menu_append (GTK_MENU (menu), gtk_separator_menu_item_new ());
 		gtk_menu_append (GTK_MENU (menu), gnome_db_new_menu_item (GTK_STOCK_SAVE_AS, TRUE,
@@ -1659,6 +1871,18 @@ confirm_file_overwrite (GtkWindow *parent, const gchar *path)
 	return yes;
 }
 
+static void
+menu_set_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
+{
+	action_filter_cb (NULL, grid);
+}
+
+static void
+menu_unset_filter_cb (GtkWidget *widget, GnomeDbRawGrid *grid)
+{
+	gda_data_proxy_set_filter_expr (grid->priv->proxy, NULL, NULL);
+}
+
 
 static void
 tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GnomeDbRawGrid *grid)
