@@ -29,6 +29,7 @@
 #include "gnome-db-data-store.h"
 #include "gnome-db-data-widget.h"
 #include "gnome-db-data-widget-info.h"
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 static void gnome_db_server_operation_class_init (GnomeDbServerOperationClass *class);
 static void gnome_db_server_operation_init (GnomeDbServerOperation *wid);
@@ -719,7 +720,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 
 	/* load Glade file for specific GUI if it exists */
 #ifdef HAVE_LIBGLADE
-	glade_file = g_build_filename (DATADIR, "gnome-db", "server_operation.glade", NULL);
+	glade_file = gnome_db_gbr_get_data_dir_path ("server_operation.glade");
 	form->priv->glade = glade_xml_new (glade_file, 
 					   gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (form->priv->op)), 
 					   NULL);
