@@ -1,6 +1,6 @@
-/* gnome-db-util.c
+/* gnome-db-tools.c
  *
- * Copyright (C) 2005 - 2006 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 - 2007 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -206,6 +206,8 @@ static GnomeDbDataEntry *entry_date_create_func (GdaDataHandler *handler, GType
 static GtkCellRenderer *cell_textual_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GtkCellRenderer *cell_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 
+static xmlChar *get_spec_with_isocodes (const gchar *file);
+
 static GHashTable *
 init_plugins_hash (void)
 {
@@ -284,13 +286,10 @@ init_plugins_hash (void)
 	plugin->cell_create_func = cell_textual_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
 	file = gnome_db_gbr_get_data_dir_path ("gnome-db-entry-string-number.xml");
-	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
-		g_message ("Could not find file '%s': '%s' data entry will not report any possible option",
-				   file, plugin->plugin_name);
-        }
-	else {
-		gsize len;
-		g_file_get_contents (file, &(plugin->options_xml_spec), &len, NULL);
+	xmlChar *xml_spec = get_spec_with_isocodes (file);
+	if (xml_spec) {
+		plugin->options_xml_spec = g_strdup (xml_spec);
+		xmlFree (xml_spec);
 	}
 	g_free (file);
 
@@ -382,7 +381,7 @@ init_plugins_hash (void)
 				continue;
 			}
 
-			g_module_symbol (handle, "plugin_init", (gpointer *) &plugin_init);
+			g_module_symbol (handle, "plugin_init", &plugin_init);
 			if (plugin_init) {
 				g_print (_("Loading file %s...\n"), path);
 				plugins = plugin_init (&err);
@@ -467,3 +466,129 @@ cell_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *opti
 {
 	return gnome_db_data_cell_renderer_boolean_new (handler, G_TYPE_BOOLEAN);
 }
+
+static xmlNodePtr
+find_child_node_from_name (xmlNodePtr parent, const gchar *name, const gchar *attr_name, const gchar *attr_value)
+{
+	xmlNodePtr node;
+
+	if (!parent)
+		return NULL;
+
+	for (node = parent->children; node; node = node->next) {
+		if (!strcmp (node->name, name)) {
+			if (attr_name) {
+				xmlChar *prop;
+				prop = xmlGetProp (node, attr_name);
+				if (prop) {
+					if (attr_value && !strcmp (prop, attr_value)) {
+						xmlFree (prop);
+						break;
+					}
+					xmlFree (prop);
+				}
+			}
+			else
+				break;
+		}
+	}
+	if (!node) 
+		g_warning ("Failed to find the <%s> tag", name);
+
+	return node;
+}
+
+static xmlChar *
+get_spec_with_isocodes (const gchar *file)
+{
+	xmlDocPtr spec, isocodes = NULL;
+	gchar *retval = NULL;
+	gchar *isofile = NULL;
+	GError *err = NULL;
+	gchar  *buf = NULL;
+	gsize   buf_len;
+	
+	/*
+	 * Load iso codes
+	 */
+#define ISO_CODES_LOCALESDIR ISO_CODES_PREFIX "/share/locale"
+
+	bindtextdomain ("iso_4217", ISO_CODES_LOCALESDIR);
+	bind_textdomain_codeset ("iso_4217", "UTF-8");
+
+	isofile = g_build_filename (ISO_CODES_PREFIX, "share", "xml", "iso-codes", "iso_4217.xml", NULL);
+	if (g_file_get_contents (isofile, &buf, &buf_len, &err)) {
+		isocodes = xmlParseDoc (BAD_CAST buf);
+		g_free (buf);
+		buf = NULL;
+	} 
+
+	/* 
+	 * Load spec string 
+	 */
+	spec = xmlParseFile (file);
+	if (!spec) {
+		g_warning ("Can't load '%s' file", file);
+		goto cleanup;
+	}
+
+	if (isocodes) {
+		/*
+		 * Merge isocodes into spec
+		 */
+		xmlNodePtr node, spec_node;
+		
+		node = find_child_node_from_name (xmlDocGetRootElement (spec), "sources", NULL, NULL);
+		node = find_child_node_from_name (node, "gda_array", "name", "currencies");
+		spec_node = find_child_node_from_name (node, "gda_array_data", NULL, NULL);
+		xmlUnlinkNode (spec_node);
+		xmlFreeNode (spec_node);
+		spec_node = xmlNewChild (node, NULL, "gda_array_data", NULL);
+		
+		node = xmlDocGetRootElement (isocodes);
+		for (node = node->children; node; node = node->next) {
+			if (!strcmp (node->name, "iso_4217_entry")) {
+				xmlChar *code, *name;
+				code = xmlGetProp (node, "letter_code");
+				name = xmlGetProp (node, "currency_name");
+				if (code && name) {
+					xmlNodePtr row;
+					row = xmlNewChild (spec_node, NULL, "gda_array_row", NULL);
+					xmlNewChild (row, NULL, "gda_value", code);
+					xmlNewChild (row, NULL, "gda_value", code);
+					xmlNewChild (row, NULL, "gda_value", dgettext ("iso_4217", name));
+				}
+				if (code)
+					xmlFree (code);
+				if (name)
+					xmlFree (name);
+			}
+		}
+	}
+	else {
+		/*
+		 * No ISO CODES found => no predefined source
+		 */
+		xmlNodePtr node;
+		node = find_child_node_from_name (xmlDocGetRootElement (spec), "sources", NULL, NULL);
+		node = find_child_node_from_name (node, "gda_array", "name", "currencies");
+		xmlUnlinkNode (node);
+		xmlFreeNode (node);
+
+		node = find_child_node_from_name (xmlDocGetRootElement (spec), "parameters", NULL, NULL);
+		node = find_child_node_from_name (xmlDocGetRootElement (spec), "parameter", "id", "CURRENCY");
+		xmlSetProp (node, "source", NULL);
+	}
+
+	xmlDocDumpMemory (spec, (xmlChar **) &retval, &buf_len);
+
+ cleanup:
+	if (spec)
+		xmlFreeDoc (spec);
+	if (isocodes)
+		xmlFreeDoc (isocodes);
+	g_free (isofile);
+	g_free (buf);
+
+	return retval;
+}
